/*
 * Copyright (C) 2011 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "tsf.h"
#include "tsf_internal.h"
#include "tsf_test.h"
#include "comment_test_types.h"

#include <unistd.h>

static void verify_foo(tsf_type_t *foo) {
    TA(tsf_type_has_comment(foo));
    TA(!strcmp(tsf_type_get_comment(foo)," this is a comment "));
    TA(tsf_type_has_name(foo));
    TA(!strcmp(tsf_type_get_name(foo),"foo"));
    TA(!tsf_type_has_version(foo));
}

static void foo_test(void) {
    verify_foo(foo__get_type());
}

static void verify_bar(tsf_type_t *bar) {
    TA(!tsf_type_has_comment(bar));
    TA(tsf_type_has_name(bar));
    TA(!strcmp(tsf_type_get_name(bar),"bar"));
    TA(tsf_type_has_version(bar));
    TA(tsf_type_get_major_version(bar)==5);
    TA(tsf_type_get_minor_version(bar)==1);
    TA(tsf_type_get_patch_version(bar)==9);
}

static void bar_test(void) {
    verify_bar(bar__get_type());
}

static void baz_test(void) {
    TA(!tsf_type_has_comment(baz__get_type()));
    TA(tsf_type_has_name(baz__get_type()));
    TA(!strcmp(tsf_type_get_name(baz__get_type()),"blah"));
    TA(tsf_type_has_version(baz__get_type()));
    TA(tsf_type_get_major_version(baz__get_type())==10);
    TA(tsf_type_get_minor_version(baz__get_type())==0);
    TA(tsf_type_get_patch_version(baz__get_type())==1);
}

static void this_test(void) {
    TA(!tsf_type_has_comment(this__get_type()));
    TA(tsf_type_has_name(this__get_type()));
    TA(!strcmp(tsf_type_get_name(this__get_type()),"bleh"));
    TA(tsf_type_has_version(this__get_type()));
    TA(tsf_type_get_major_version(this__get_type())==9);
    TA(tsf_type_get_minor_version(this__get_type())==7);
    TA(tsf_type_get_patch_version(this__get_type())==2);
}

static tsf_stream_file_input_t *input;
static tsf_stream_file_output_t *output;

static void t_reset(void) {
    if (1) unlink("comment_test.dat");
}

static void t_open_output(void) {
    TAT(output=tsf_stream_file_output_open("comment_test.dat",tsf_false,NULL));
}

static void t_close_output(void) {
    TAT(tsf_stream_file_output_close(output));
}

static void t_open_input(void) {
    TAT(input=tsf_stream_file_input_open("comment_test.dat",NULL,NULL));
}

static void t_close_input(void) {
    tsf_stream_file_input_close(input);
}

static void foo_write_read_test(void) {
    foo_t data,*datap;
    tsf_buffer_t *buf;
    
    data.a=42;
    
    t_reset();
    
    t_open_output();
    TAT(foo__write(output,&data));
    t_close_output();
    
    t_open_input();
    TAT(buf=tsf_stream_file_input_read(input));
    TAT(datap=foo__parse(buf));
    TA(tsf_type_compare(tsf_buffer_get_type(buf),foo__get_type()));
    verify_foo(tsf_buffer_get_type(buf));
    TA(datap->a==42);
    tsf_region_free(datap);
    tsf_buffer_destroy(buf);
    t_close_input();
    
    t_reset();
}

static void bar_write_read_test(void) {
    bar_t data,*datap;
    tsf_buffer_t *buf;
    
    data.b=(float)2.;
    
    t_reset();
    
    t_open_output();
    TAT(bar__write(output,&data));
    t_close_output();
    
    t_open_input();
    TAT(buf=tsf_stream_file_input_read(input));
    TAT(datap=bar__parse(buf));
    TA(tsf_type_compare(tsf_buffer_get_type(buf),bar__get_type()));
    verify_bar(tsf_buffer_get_type(buf));
    TA(datap->b==(float)2.);
    tsf_region_free(datap);
    tsf_buffer_destroy(buf);
    t_close_input();

    t_reset();
}

int main(int c,char **v) {
    TEST_INIT(c,v);
    
    DO_TEST(foo_test());
    DO_TEST(bar_test());
    DO_TEST(baz_test());
    DO_TEST(this_test());
    DO_TEST(foo_write_read_test());
    DO_TEST(bar_write_read_test());
    
    TEST_DONE();
    
    return 0;
}

