/*
 * Copyright (C) 2014 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "tsf_test.h"
#include "tsf_internal.h"
#include "write_cases.h"

#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

static char *directory_name;

static void doit(tsf_reflect_t *r) {
    tsf_buffer_t *buf;
    char *filename;
    char *filename_with_hash;
    tsf_stream_file_output_t *output;
    tsf_sha1_wtr_t *sha1_writer;
    int fd;
    char *sha1_digest;
    
    TAT(buf = tsf_reflect_make_buffer(r));
    tsf_reflect_destroy(r);

    filename = tsf_asprintf("%s/unhashed_model.tsf", directory_name);
    TAT(output = tsf_stream_file_output_open(filename, tsf_false, NULL));
    TAT(tsf_stream_file_output_write(output, buf));
    tsf_buffer_destroy(buf);
    TAT(tsf_stream_file_output_close(output));
    
    TAT(sha1_writer = tsf_sha1_writer_create());
    TAE((fd = open(filename, O_RDONLY)) >= 0);
    for (;;) {
        char buf[1024];
        int res;
        TAE((res = read(fd, buf, sizeof(buf))) >= 0);
        if (res == 0)
            break;
        TAT(tsf_sha1_writer_write(sha1_writer, buf, res));
    }
    
    TAT(sha1_digest = tsf_sha1_sum_to_str(tsf_sha1_writer_finish(sha1_writer)));
    tsf_sha1_writer_destroy(sha1_writer);
    
    filename_with_hash = tsf_asprintf("%s/model_%s.tsf", directory_name, sha1_digest);
    free(sha1_digest);
    
    TAE(rename(filename, filename_with_hash) == 0);
    free(filename);
    free(filename_with_hash);
}

int main(int c,char **v) {
    if (c<3) {
        fprintf(stderr, "Usage: %s <output directory> <use comments>\n", v[0]);
        return 1;
    }
    
    directory_name = strdup(v[1]);
    if (mkdir(directory_name, 0777) < 0) {
        TAE(errno == EEXIST);
    }
    
    TEST_INIT_NOFORK(c,v);
    
    do_tests(tolower(v[2][0]) == 't', doit);
    
    TEST_DONE_CUSTOM(" Done.");
    
    return 0;
}
