/*
 * Copyright (C) 2004, 2005 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

typedef tsf_reflect_t create_data_t;

static tsf_buffer_t *create_make_buffer(tsf_type_t *type,
                                        tsf_reflect_t *r) {
    tsf_buffer_t *buf;
    TAT(buf=tsf_reflect_make_buffer(r));
    return buf;
}

static void create_destroy_data(tsf_reflect_t *r) {
    tsf_reflect_destroy(r);
}

#define DEFINE_PRIMITIVE_CREATE_FUNCS(type_lc_name, type_lc, type_uc)   \
    static tsf_reflect_t *create_##type_lc_name(type_lc value) {        \
        tsf_reflect_t *r;                                               \
        TAT(r = tsf_reflect_create_##type_lc_name(value));              \
        return r;                                                       \
    }                                                                   \
    typedef int define_primitive_create_funcs_##type_lc_name

static tsf_reflect_t *create_bit(tsf_bool_t value) {
    tsf_reflect_t *r;
    TAT(r=tsf_reflect_create_bit(value));
    return r;
}

static tsf_reflect_t *create_void(void) {
    tsf_reflect_t *r;
    TAT(r=tsf_reflect_create_void());
    return r;
}

static tsf_reflect_t *create_string(uint32_t size) {
    tsf_reflect_t *r;
    char *string;
    uint32_t i;
    TAE(string=malloc(size+1));
    for (i=0;i<size;++i) {
        string[i]=(i%255)+1;
    }
    string[size]=0;
    TAT(r=tsf_reflect_create_string(string));
    TA(!strcmp(tsf_string_reflect_get(r),string));
    return r;
}

static tsf_reflect_t *create_any(tsf_buffer_t *sub) {
    tsf_reflect_t *r;
    TAT(r=tsf_reflect_create_any(sub));
    return r;
}

static tsf_reflect_t *create_array(tsf_type_t *ele_type,
                                   uint32_t total_elements,
                                   uint32_t num_elements,
                                   tsf_reflect_t **ele) {
    tsf_reflect_t *r;
    uint32_t n,i=0;
    TAT(r=tsf_reflect_create(tsf_type_create_array(ele_type)));
    for (n=total_elements;n-->0;) {
        TAT(tsf_array_reflect_append(r,tsf_reflect_clone(ele[i])));
        i=((i+1)%num_elements);
    }
    TA(tsf_array_reflect_get_num_elements(r)==total_elements);
    return r;
}

static tsf_reflect_t *create_struct(tsf_type_t *type,
                                    tsf_reflect_t **ele) {
    uint32_t i;
    tsf_reflect_t *r;
    TAT(r=tsf_reflect_create(type));
    for (i=0;i<tsf_struct_type_get_num_elements(type);++i) {
        TAT(tsf_struct_reflect_set_element_by_index(r,
                                                    i,
                                                    ele[i]));
    }
    return r;
}

static tsf_reflect_t *create_choice(tsf_type_t *type,
                                    uint32_t value,
                                    tsf_reflect_t *data) {
    tsf_reflect_t *r;
    TAT(r=tsf_reflect_create(type));
    TAT(tsf_choice_reflect_set_by_index(r,
                                        value,
                                        data));
    return r;
}



