/*
 * Copyright (C) 2004, 2005 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

typedef tsf_reflect_t verify_data_t;

static tsf_reflect_t *verify_from_buffer(tsf_type_t *type,
                                         tsf_buffer_t *buf) {
    tsf_reflect_t *r;
    TAT(r=tsf_reflect_from_buffer(buf));
    return r;
}

static void verify_destroy_data(tsf_reflect_t *data) {
    tsf_reflect_destroy(data);
}

#define DEFINE_PRIMITIVE_VERIFY_FUNCS(type_lc_name, type_lc, type_uc)   \
    static void verify_##type_lc_name(tsf_reflect_t *r, type_lc value) { \
        TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))              \
           == type_uc);                                                 \
        TA(tsf_primitive_reflect_get(r, type_lc_name) == value);        \
    }                                                                   \
    typedef int define_primitive_verify_funcs_##type_lc_name

static void verify_bit(tsf_reflect_t *r,tsf_bool_t value) {
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_BIT);
    TA(tsf_bit_reflect_get(r)==value);
}

static void verify_void(tsf_reflect_t *r) {
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_VOID);
}

static void verify_string(tsf_reflect_t *r,
                          uint32_t size) {
    const char *string;
    uint32_t i;
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_STRING);
    string=tsf_string_reflect_get(r);
    TA(string!=NULL);
    TA(strlen(string)==size);
    for (i=0;i<size;++i) {
        TA(((unsigned char)string[i])==(i%255)+1);
    }
}

static tsf_buffer_t *verify_any(tsf_reflect_t *r) {
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_ANY);
    return tsf_any_reflect_get_buffer(r);
}

static void verify_array(tsf_type_t *t, /* this is the type of the array, not
                                           the type of the array element. */
                         tsf_reflect_t *r,
                         uint32_t total_elements,
                         uint32_t num_elements,
                         void (*cback)(tsf_reflect_t *r,
                                       uint32_t i,
                                       void *arg),
                         void *arg) {
    uint32_t i;
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_ARRAY);
    TA(tsf_array_reflect_get_num_elements(r)==total_elements);
    for (i=0;i<total_elements;++i) {
        cback(tsf_array_reflect_get_element(r,i),
              i%num_elements,
              arg);
    }
}

static void verify_struct(tsf_type_t *t,
                          tsf_reflect_t *r,
                          void (*cback)(tsf_reflect_t *r,
                                        uint32_t i,
                                        void *arg),
                          void *arg) {
    uint32_t i;
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_STRUCT);
    for (i=0;i<tsf_struct_type_get_num_elements(t);++i) {
        cback(tsf_struct_reflect_get_element_by_index(r,i),
              i,
              arg);
    }
}

static tsf_reflect_t *verify_choice(tsf_type_t *t,
                                    tsf_reflect_t *r,
                                    uint32_t value) {
    TA(tsf_type_get_kind_code(tsf_reflect_get_type(r))
       ==TSF_TK_CHOICE);
    TA(tsf_choice_reflect_get_index(r)
       ==value);
    if (value!=UINT32_MAX) {
        return tsf_choice_reflect_get_data(r);
    }
    return NULL;
}


