/*
 * portable_param.t -- TSF definition for output generated by ql_param
 * by Filip Pizlo, 2005, 2006, 2007
 */

declare portable_command_line: array of string

declare portable_stereo_header: struct {
	run_number:			uint32

	commands: array of struct {
		/** the settings used for parameterization */
		optional settings: choice {
			none: void
			some: string
		}
	
		/** the command-line */
		optional command_line: choice {
			none: void
			some: portable_command_line
		}
	}
}

declare portable_mono_evinfo: struct {
	event_number:		uint32
}

declare portable_stereo_evinfo: struct {
	event_number:		uint32
}

declare portable_channel: struct {
	raw_signal: float
	toff: uint16
	width: uint16
	signal: float
	signal_to_noise: float
	light: float
	clean: choice {
		notknown: void
		disabled: void
		notimage: void
		picture: void
		boundary: void
	}
}

declare portable_channel_array: array of portable_channel

declare portable_gps_time: choice {
    none: void
    some: struct {
	time: double
	year: uint8
    }
}

/** hillas parameters. */
// maybe I should remove some of these parameters?  I don't cut on them
// or plot them anyway...
declare portable_mono_param: struct {
	
	derotation:		        float
	max1:				float
	max2:				float
	max3:				float
	loc1:				uint16
	loc2:				uint16
	loc3:				uint16
	nimage:				uint16
	size:				float
	xc:				float
	yc:				float
	dist:				float
	length:				float
	width:				float
	miss:				float
	cos_psi:			float
	sin_psi:			float
	azwidth:			float
	asymmetry:			float
	minor_asymmetry: 		float
	alpha:				float
	sin_alpha:			float
	length_over_size:		float
	impact_dist:			float
	impact_coeff:			float
	stereo_accept:			bit
}

declare portable_point: struct {
	x:		float
	y:		float
}

declare portable_horizon: struct {
        alt: float
        az: float
}

declare portable_equatorial: struct {
	ra: float
	dec: float
}

declare portable_stereo_param: struct {
	gps_time: portable_gps_time

	has_pointing:           bit
	pointing:               portable_horizon
	derotation:             float
	n_accept_tels:          uint8
	source:			portable_point
	source_dr:		portable_point
        source_condest: 	float
	source_chi2:            float
        source_max_angle:       float
	core:			portable_point
        core_condest:  		float
	core_chi2:		float
	n_msw:			uint8
	msw:			float
	n_msl:			uint8
	msl:			float
	theta:			float
}

declare portable_simulation_data: struct {
	id: int32
	
	/** energy in GeV */
	energy: float
	
	observation: portable_horizon
	
	primary: portable_horizon
	
	core: portable_point
}

declare portable_stereo_param_entry: struct {
	evinfo:			portable_stereo_evinfo

	trig_mask:		choice {
		none:   		void
		some:			array of bit
	}
	
	l3_gps_time:            portable_gps_time

	present_mask:		array of bit

	mono_params:		array of struct {
		telescope_id:		uint8
		
		gps_time:               portable_gps_time
	
		optional pointing:      choice {
			none:		void
			some:		portable_horizon
		}

		/** if the parameter is missing, it means that this
		    parameter wasn't computed at all, and wouldn't
		    have been used towards computing stereo parameters */
		optional param:		choice {
			none:		void
			some:		portable_mono_param
		}

		// not sure that this is the best place for this.
		// I want to do this in such a way that it is easy
		// to have files that only have channel data,
		// without any actual parameters
		optional channel_data:  choice {
			none: void
			some: portable_channel_array
		}
	}
	
	/** if the stereo param is missing, it means that computing it
	    failed, or the user asked that it not be computed. */
	optional stereo_param:	choice {
		none:		void
		some:		portable_stereo_param
	}

	optional sim_data:	choice {
		none:			void
		some:			portable_simulation_data
	}
}

declare portable_thresholds: struct {
	picture_index:		uint8
	boundary_index:		uint8
}

declare portable_mono_param_multiclean_entry: struct {
	cleaning:			array of portable_thresholds

	param:				portable_mono_param
}

declare portable_mono_param_simple_entry: struct {
	param:				portable_mono_param
}

declare portable_mean_trace_channel: array of uint64

declare portable_mean_trace: struct {
	telescope_id: uint8
	
	traces: array of portable_mean_trace_channel
	n: uint64

	num_samples: uint32
}

declare portable_stats: struct {
	n: double
	sum: double
	sum_2: double
}

declare portable_charge_map: struct {
	telescope_id: uint8
	channels: array of portable_stats
}

declare portable_valdev: struct {
	val: double
	dev: double
	masked: bit
	masks: array of string
}

declare portable_pedestals_channels: array of portable_valdev

declare portable_pedestals: struct {
	telescope_id: uint8
	channels: portable_pedestals_channels
	pair_channels: choice {
		none: void
		some: portable_pedestals_channels
	}
}

