/*
 * Copyright (C) 2005, 2011 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <sys/file.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

static void usage() {
    printf("Usage: tsf_flock shared|exclusive <lock file> <shell command>\n");
    printf("   or: tsf_flock shared|exclusive <lock file>\n");
    printf("Description:\n");
    printf("Acquires either a shared or exclusive lock on the given file and runs the\n");
    printf("given shell command.  If the shell command is not given, this program will\n");
    printf("wait simply pause while holding the lock until it is terminated.\n");
    exit(1);
}

int main(int c,char **v) {
    int fd;

    if (c!=3 && c!=4) {
	usage();
    }

    if (strcmp(v[1],"shared") &&
	strcmp(v[1],"exclusive")) {
	fprintf(stderr,"First argument must be 'shared' or 'exclusive'.\n");
	usage();
    }

    fd=open(v[2],O_RDWR);
    if (fd<0) {
	fprintf(stderr,"Could not open %s: %s\n",
		v[2],strerror(errno));
	return 1;
    }

    if (flock(fd,strcmp(v[1],"shared")?LOCK_EX:LOCK_SH)<0) {
	fprintf(stderr,"Could not acquire lock on %s: %s\n",
		v[2],strerror(errno));
	return 1;
    }
    
    if (c==4) {
	int res=system(v[3]);
	if (res<0) {
	    fprintf(stderr,"Got an error in system(): %s\n",
		    strerror(errno));
	    return 1;
	}
	return res;
    }
    
    pause();
    return 0;
}




